"""
Wrapper for newrelic api for making nrql requests

"""
__author__ = 'Vladimir Elizarov'
__maintainer__ = __author__

__email__ = 'vladimirelizarov89@gmail.com'
__license__ = 'MIT'
__version__ = '0.0.3'


__all__ = (
    '__author__',
    '__email__',
    '__license__',
    '__maintainer__',
    '__version__',
)